@@ -77,7 +77,7 @@ class DFPlayer : public uart::UARTDevice, public Component {
   class ACTION_CLASS : /* NOLINT */ \
                        public Action<Ts...>, \
                        public Parented<DFPlayer> { \
-    void play(Ts... x) override { this->parent_->ACTION_METHOD(); } \
+    void play(const Ts &...x) override { this->parent_->ACTION_METHOD(); } \
   };
 
 DFPLAYER_SIMPLE_ACTION(NextAction, next)
@@ -87,7 +87,7 @@ template<typename... Ts> class PlayMp3Action : public Action<Ts...>, public Pare
  public:
   TEMPLATABLE_VALUE(uint16_t, file)
 
-  void play(Ts... x) override {
+  void play(const Ts &...x) override {
     auto file = this->file_.value(x...);
     this->parent_->play_mp3(file);
   }
@@ -98,7 +98,7 @@ template<typename... Ts> class PlayFileAction : public Action<Ts...>, public Par
   TEMPLATABLE_VALUE(uint16_t, file)
   TEMPLATABLE_VALUE(bool, loop)
 
-  void play(Ts... x) override {
+  void play(const Ts &...x) override {
     auto file = this->file_.value(x...);
     auto loop = this->loop_.value(x...);
     if (loop) {
@@ -115,7 +115,7 @@ template<typename... Ts> class PlayFolderAction : public Action<Ts...>, public P
   TEMPLATABLE_VALUE(uint16_t, file)
   TEMPLATABLE_VALUE(bool, loop)
 
-  void play(Ts... x) override {
+  void play(const Ts &...x) override {
     auto folder = this->folder_.value(x...);
     auto file = this->file_.value(x...);
     auto loop = this->loop_.value(x...);
@@ -131,7 +131,7 @@ template<typename... Ts> class SetDeviceAction : public Action<Ts...>, public Pa
  public:
   TEMPLATABLE_VALUE(Device, device)
 
-  void play(Ts... x) override {
+  void play(const Ts &...x) override {
     auto device = this->device_.value(x...);
     this->parent_->set_device(device);
   }
@@ -141,7 +141,7 @@ template<typename... Ts> class SetVolumeAction : public Action<Ts...>, public Pa
  public:
   TEMPLATABLE_VALUE(uint8_t, volume)
 
-  void play(Ts... x) override {
+  void play(const Ts &...x) override {
     auto volume = this->volume_.value(x...);
     this->parent_->set_volume(volume);
   }
@@ -151,7 +151,7 @@ template<typename... Ts> class SetEqAction : public Action<Ts...>, public Parent
  public:
   TEMPLATABLE_VALUE(EqPreset, eq)
 
-  void play(Ts... x) override {
+  void play(const Ts &...x) override {
     auto eq = this->eq_.value(x...);
     this->parent_->set_eq(eq);
   }
@@ -168,7 +168,7 @@ DFPLAYER_SIMPLE_ACTION(VolumeDownAction, volume_down)
 
 template<typename... Ts> class DFPlayerIsPlayingCondition : public Condition<Ts...>, public Parented<DFPlayer> {
  public:
-  bool check(Ts... x) override { return this->parent_->is_playing(); }
+  bool check(const Ts &...x) override { return this->parent_->is_playing(); }
 };
 
 class DFPlayerFinishedPlaybackTrigger : public Trigger<> {
