@@ -16,13 +16,28 @@ void GreeClimate::set_model(Model model) {
   this->model_ = model;
 }
 
+void GreeClimate::set_mode_bit(uint8_t bit_mask, bool enabled) {
+  if (enabled) {
+    this->mode_bits_ |= bit_mask;
+  } else {
+    this->mode_bits_ &= ~bit_mask;
+  }
+  this->transmit_state();
+}
+
 void GreeClimate::transmit_state() {
   uint8_t remote_state[8] = {0x00, 0x00, 0x00, 0x00, 0x00, 0x20, 0x00, 0x00};
 
   remote_state[0] = this->fan_speed_() | this->operation_mode_();
   remote_state[1] = this->temperature_();
 
-  if (this->model_ == GREE_YAN || this->model_ == GREE_YX1FF || this->model_ == GREE_YAG) {
+  if (this->model_ == GREE_YAN) {
+    remote_state[2] = 0x20;  // bits 0..3 always 0000, bits 4..7 TURBO, LIGHT, HEALTH, X-FAN
+    remote_state[3] = 0x50;  // bits 4..7 always 0101
+    remote_state[4] = this->vertical_swing_();
+  }
+
+  if (this->model_ == GREE_YX1FF || this->model_ == GREE_YAG) {
     remote_state[2] = 0x60;
     remote_state[3] = 0x50;
     remote_state[4] = this->vertical_swing_();
@@ -41,7 +56,7 @@ void GreeClimate::transmit_state() {
   }
 
   if (this->model_ == GREE_YAA || this->model_ == GREE_YAC || this->model_ == GREE_YAC1FB9) {
-    remote_state[2] = 0x20;  // bits 0..3 always 0000, bits 4..7 TURBO,LIGHT,HEALTH,X-FAN
+    remote_state[2] = 0x20;  // bits 0..3 always 0000, bits 4..7 TURBO, LIGHT, HEALTH, X-FAN
     remote_state[3] = 0x50;  // bits 4..7 always 0101
     remote_state[6] = 0x20;  // YAA1FB, FAA1FB1, YB1F2 bits 4..7 always 0010
 
@@ -52,6 +67,13 @@ void GreeClimate::transmit_state() {
     }
   }
 
+  if (this->model_ == GREE_YAN || this->model_ == GREE_YAA || this->model_ == GREE_YAC ||
+      this->model_ == GREE_YAC1FB9) {
+    // Merge the mode bits into remote_state[2]
+    // Clear the mode bits (bits 4-7) and OR in the current mode_bits_
+    remote_state[2] = (remote_state[2] & 0x0F) | this->mode_bits_;
+  }
+
   if (this->model_ == GREE_YX1FF) {
     if (this->fan_speed_() == GREE_FAN_TURBO) {
       remote_state[2] |= GREE_FAN_TURBO_BIT;
