@@ -1,10 +1,8 @@
 #pragma once
 
-#include <map>
-#include <string>
-
 #include "esphome/components/number/number.h"
 #include "esphome/core/component.h"
+#include "esphome/core/string_ref.h"
 
 namespace esphome {
 namespace homeassistant {
@@ -18,10 +16,10 @@ class HomeassistantNumber : public number::Number, public Component {
   float get_setup_priority() const override;
 
  protected:
-  void state_changed_(const std::string &state);
-  void min_retrieved_(const std::string &min);
-  void max_retrieved_(const std::string &max);
-  void step_retrieved_(const std::string &step);
+  void state_changed_(StringRef state);
+  void min_retrieved_(StringRef min);
+  void max_retrieved_(StringRef max);
+  void step_retrieved_(StringRef step);
 
   void control(float value) override;
 
