@@ -36,8 +36,8 @@ class I2SAudioMicrophone : public I2SAudioIn, public microphone::Microphone, pub
 
 #ifdef USE_I2S_LEGACY
 #if SOC_I2S_SUPPORTS_ADC
-  void set_adc_channel(adc1_channel_t channel) {
-    this->adc_channel_ = channel;
+  void set_adc_channel(adc_channel_t channel) {
+    this->adc_channel_ = (adc1_channel_t) channel;
     this->adc_ = true;
   }
 #endif
