@@ -3,8 +3,7 @@
 #include "esphome/components/button/button.h"
 #include "../ld2450.h"
 
-namespace esphome {
-namespace ld2450 {
+namespace esphome::ld2450 {
 
 class FactoryResetButton : public button::Button, public Parented<LD2450Component> {
  public:
@@ -14,5 +13,4 @@ class FactoryResetButton : public button::Button, public Parented<LD2450Componen
   void press_action() override;
 };
 
-}  // namespace ld2450
-}  // namespace esphome
+}  // namespace esphome::ld2450
