@@ -3,8 +3,7 @@
 #include "esphome/components/select/select.h"
 #include "../ld2450.h"
 
-namespace esphome {
-namespace ld2450 {
+namespace esphome::ld2450 {
 
 class BaudRateSelect : public select::Select, public Parented<LD2450Component> {
  public:
@@ -14,5 +13,4 @@ class BaudRateSelect : public select::Select, public Parented<LD2450Component> {
   void control(size_t index) override;
 };
 
-}  // namespace ld2450
-}  // namespace esphome
+}  // namespace esphome::ld2450
