@@ -32,8 +32,10 @@ from .const import (
     CONF_SDK_SILENT,
     CONF_UART_PORT,
     FAMILIES,
+    FAMILY_BK7231N,
     FAMILY_COMPONENT,
     FAMILY_FRIENDLY,
+    FAMILY_RTL8710B,
     KEY_BOARD,
     KEY_COMPONENT,
     KEY_COMPONENT_DATA,
@@ -50,6 +52,22 @@ CODEOWNERS = ["@kuba2k2"]
 AUTO_LOAD = ["preferences"]
 IS_TARGET_PLATFORM = True
 
+# BK7231N SDK options to disable unused features.
+# Disabling BLE saves ~21KB RAM and ~200KB Flash because BLE init code is
+# called unconditionally by the SDK. ESPHome doesn't use BLE on LibreTiny.
+#
+# This only works on BK7231N (BLE 5.x). Other BK72XX chips using BLE 4.2
+# (BK7231T, BK7231Q, BK7251; BK7252 boards use the BK7251 family) have a bug
+# where the BLE library still links and references undefined symbols when
+# CFG_SUPPORT_BLE=0.
+#
+# Other options like CFG_TX_EVM_TEST, CFG_RX_SENSITIVITY_TEST, CFG_SUPPORT_BKREG,
+# CFG_SUPPORT_OTA_HTTP, and CFG_USE_SPI_SLAVE were evaluated but provide no  # NOLINT
+# measurable benefit - the linker already strips unreferenced code via -gc-sections.
+_BK7231N_SYS_CONFIG_OPTIONS = [
+    "CFG_SUPPORT_BLE=0",
+]
+
 
 def _detect_variant(value):
     if KEY_LIBRETINY not in CORE.data:
@@ -174,9 +192,9 @@ def _notify_old_style(config):
 
 # The dev and latest branches will be at *least* this version, which is what matters.
 ARDUINO_VERSIONS = {
-    "dev": (cv.Version(1, 9, 1), "https://github.com/libretiny-eu/libretiny.git"),
-    "latest": (cv.Version(1, 9, 1), "libretiny"),
-    "recommended": (cv.Version(1, 9, 1), None),
+    "dev": (cv.Version(1, 9, 2), "https://github.com/libretiny-eu/libretiny.git"),
+    "latest": (cv.Version(1, 9, 2), "libretiny"),
+    "recommended": (cv.Version(1, 9, 2), None),
 }
 
 
@@ -261,11 +279,23 @@ async def component_to_code(config):
     cg.add_build_flag(f"-DUSE_LIBRETINY_VARIANT_{config[CONF_FAMILY]}")
     cg.add_define("ESPHOME_BOARD", config[CONF_BOARD])
     cg.add_define("ESPHOME_VARIANT", FAMILY_FRIENDLY[config[CONF_FAMILY]])
-    # LibreTiny uses MULTI_NO_ATOMICS because platforms like BK7231N (ARM968E-S) lack
-    # exclusive load/store (no LDREX/STREX). std::atomic RMW operations require libatomic,
-    # which is not linked to save flash (4-8KB). Even if linked, libatomic would use locks
-    # (ATOMIC_INT_LOCK_FREE=1), so explicit FreeRTOS mutexes are simpler and equivalent.
-    cg.add_define(ThreadModel.MULTI_NO_ATOMICS)
+    # Set threading model based on chip architecture
+    component: LibreTinyComponent = CORE.data[KEY_LIBRETINY][KEY_COMPONENT_DATA]
+    if component.supports_atomics:
+        # RTL87xx (Cortex-M4) and LN882x (Cortex-M4F) have LDREX/STREX
+        cg.add_define(ThreadModel.MULTI_ATOMICS)
+    else:
+        # BK72xx uses ARM968E-S (ARMv5TE) which lacks LDREX/STREX.
+        # std::atomic RMW operations would require libatomic (not linked to save
+        # 4-8KB flash). Even if linked, it would use locks, so explicit FreeRTOS
+        # mutexes are simpler and equivalent.
+        cg.add_define(ThreadModel.MULTI_NO_ATOMICS)
+
+    # RTL8710B needs FreeRTOS 8.2.3+ for xTaskNotifyGive/ulTaskNotifyTake
+    # required by AsyncTCP 3.4.3+ (https://github.com/esphome/esphome/issues/10220)
+    # RTL8720C (ambz2) requires FreeRTOS 10.x so this only applies to RTL8710B
+    if config[CONF_FAMILY] == FAMILY_RTL8710B:
+        cg.add_platformio_option("custom_versions.freertos", "8.2.3")
 
     # force using arduino framework
     cg.add_platformio_option("framework", "arduino")
@@ -346,4 +376,10 @@ async def component_to_code(config):
         cg.add_platformio_option("custom_fw_name", "esphome")
         cg.add_platformio_option("custom_fw_version", __version__)
 
+    # Apply chip-specific SDK options to save RAM/Flash
+    if config[CONF_FAMILY] == FAMILY_BK7231N:
+        cg.add_platformio_option(
+            "custom_options.sys_config#h", _BK7231N_SYS_CONFIG_OPTIONS
+        )
+
     await cg.register_component(var, config)
