@@ -9,7 +9,7 @@ namespace mixer_speaker {
 template<typename... Ts> class DuckingApplyAction : public Action<Ts...>, public Parented<SourceSpeaker> {
   TEMPLATABLE_VALUE(uint8_t, decibel_reduction)
   TEMPLATABLE_VALUE(uint32_t, duration)
-  void play(Ts... x) override {
+  void play(const Ts &...x) override {
     this->parent_->apply_ducking(this->decibel_reduction_.value(x...), this->duration_.value(x...));
   }
 };
