@@ -1,10 +1,8 @@
 #include "esphome/core/log.h"
 #include "number_traits.h"
 
-namespace esphome {
-namespace number {
+namespace esphome::number {
 
 static const char *const TAG = "number";
 
-}  // namespace number
-}  // namespace esphome
+}  // namespace esphome::number
