@@ -1,11 +1,7 @@
+from esphome import pins
 import esphome.codegen as cg
 import esphome.config_validation as cv
-from esphome import pins
-from esphome.const import (
-    CONF_CLOCK_PIN,
-    CONF_DATA_PIN,
-    CONF_ID,
-)
+from esphome.const import CONF_CLOCK_PIN, CONF_DATA_PIN, CONF_ID
 
 CODEOWNERS = ["@Cossid"]
 MULTI_CONF = True
