@@ -14,11 +14,13 @@ void SM16716::setup() {
   this->pwm_amounts_.resize(this->num_channels_, 0);
 }
 void SM16716::dump_config() {
-  ESP_LOGCONFIG(TAG, "SM16716:");
+  ESP_LOGCONFIG(TAG,
+                "SM16716:\n"
+                "  Total number of channels: %u\n"
+                "  Number of chips: %u",
+                this->num_channels_, this->num_chips_);
   LOG_PIN("  Data Pin: ", this->data_pin_);
   LOG_PIN("  Clock Pin: ", this->clock_pin_);
-  ESP_LOGCONFIG(TAG, "  Total number of channels: %u", this->num_channels_);
-  ESP_LOGCONFIG(TAG, "  Number of chips: %u", this->num_chips_);
 }
 void SM16716::loop() {
   if (!this->update_)
