@@ -1,11 +1,7 @@
 import esphome.codegen as cg
+from esphome.components import light, uart
 import esphome.config_validation as cv
-from esphome.components import uart, light
-from esphome.const import (
-    CONF_OUTPUT_ID,
-    CONF_MIN_VALUE,
-    CONF_MAX_VALUE,
-)
+from esphome.const import CONF_MAX_VALUE, CONF_MIN_VALUE, CONF_OUTPUT_ID
 
 CONF_USE_RM433_REMOTE = "use_rm433_remote"
 
