@@ -1,8 +1,7 @@
 #include "template_binary_sensor.h"
 #include "esphome/core/log.h"
 
-namespace esphome {
-namespace template_ {
+namespace esphome::template_ {
 
 static const char *const TAG = "template.binary_sensor";
 
@@ -23,5 +22,4 @@ void TemplateBinarySensor::loop() {
 
 void TemplateBinarySensor::dump_config() { LOG_BINARY_SENSOR("", "Template Binary Sensor", this); }
 
-}  // namespace template_
-}  // namespace esphome
+}  // namespace esphome::template_
