@@ -4,8 +4,7 @@
 
 #include "esphome/core/automation.h"
 
-namespace esphome {
-namespace template_ {
+namespace esphome::template_ {
 
 template<typename... Ts> class TemplateLockPublishAction : public Action<Ts...>, public Parented<TemplateLock> {
  public:
@@ -14,5 +13,4 @@ template<typename... Ts> class TemplateLockPublishAction : public Action<Ts...>,
   void play(const Ts &...x) override { this->parent_->publish_state(this->state_.value(x...)); }
 };
 
-}  // namespace template_
-}  // namespace esphome
+}  // namespace esphome::template_
