@@ -73,12 +73,15 @@ void TMP1075Sensor::set_fault_count(const int faults) {
 }
 
 void TMP1075Sensor::log_config_() {
-  ESP_LOGV(TAG, "  oneshot   : %d", config_.fields.oneshot);
-  ESP_LOGV(TAG, "  rate      : %d", config_.fields.rate);
-  ESP_LOGV(TAG, "  faults    : %d", config_.fields.faults);
-  ESP_LOGV(TAG, "  polarity  : %d", config_.fields.polarity);
-  ESP_LOGV(TAG, "  alert_mode: %d", config_.fields.alert_mode);
-  ESP_LOGV(TAG, "  shutdown  : %d", config_.fields.shutdown);
+  ESP_LOGV(TAG,
+           "  oneshot   : %d\n"
+           "  rate      : %d\n"
+           "  faults    : %d\n"
+           "  polarity  : %d\n"
+           "  alert_mode: %d\n"
+           "  shutdown  : %d",
+           config_.fields.oneshot, config_.fields.rate, config_.fields.faults, config_.fields.polarity,
+           config_.fields.alert_mode, config_.fields.shutdown);
 }
 
 void TMP1075Sensor::write_config() {
